Ddoc

$(SPEC_S Types,

<h2>Basic Data Types</h2>

	$(TABLE1
	<tr>
	<th>Keyword
	<th>Description
	<th>Default Initializer (.init)
	<tr><td>$(TT void)		<td>no type		<td> -
	<tr><td>$(TT bool)		<td>boolean value	<td> false
	<tr><td>$(TT byte)		<td>signed 8 bits	<td> 0
	<tr><td>$(TT ubyte)		<td>unsigned 8 bits	<td> 0
	<tr><td>$(TT short)		<td>signed 16 bits	<td> 0
	<tr><td>$(TT ushort)		<td>unsigned 16 bits	<td> 0
	<tr><td>$(TT int)		<td>signed 32 bits	<td> 0
	<tr><td>$(TT uint)		<td>unsigned 32 bits	<td> 0
	<tr><td>$(TT long)		<td>signed 64 bits	<td> 0L
	<tr><td>$(TT ulong)		<td>unsigned 64 bits	<td> 0L
	<tr><td>$(TT cent)		<td>signed 128 bits (reserved for future use)	<td> 0
	<tr><td>$(TT ucent)		<td>unsigned 128 bits (reserved for future use)	<td> 0
	<tr><td>$(TT float)		<td>32 bit floating point	<td> float.nan
	<tr><td>$(TT double)		<td>64 bit floating point	<td> double.nan
	<tr><td>$(TT real)
		<td>largest hardware implemented floating
		point size ($(B Implementation Note:) 80 bits for Intel CPUs)
		<td> real.nan
	<tr><td>$(TT ifloat)
		<td>imaginary float
		<td>float.nan * 1.0i
	<tr><td>$(TT idouble)
		<td>imaginary double
		<td>double.nan * 1.0i
	<tr><td>$(TT ireal)
		<td>imaginary real
		<td>real.nan * 1.0i
	<tr><td>$(TT cfloat)
		<td>a complex number of two float values
		<td>float.nan + float.nan * 1.0i
	<tr><td>$(TT cdouble)
		<td>complex double
		<td>double.nan + double.nan * 1.0i
	<tr><td>$(TT creal)
		<td>complex real
		<td>real.nan + real.nan * 1.0i

	<tr><td>$(TT char)	<td>unsigned 8 bit UTF-8	<td>0xFF
	<tr><td>$(TT wchar)	<td>unsigned 16 bit UTF-16	<td>0xFFFF
	<tr><td>$(TT dchar)	<td>unsigned 32 bit UTF-32	<td>0x0000FFFF
	)


<h2>Derived Data Types</h2>

    $(UL 
	<li>pointer
	<li>array
	<li>associative array
	<li>function
	<li>delegate
    )

<h2>User Defined Types</h2>

    $(UL 
	<li>alias
	<li>typedef
	<li>enum
	<li>struct
	<li>union
	<li>class
    )

<h2>Base Types</h2>

	$(P The $(I base type) of an enum is the type it is based on:)

---
enum E : T { ... }	// T is the $(I base type) of E
---

	$(P The $(I base type) of a typedef is the type it is formed from:)

---
typedef T U;		// T is the $(I base type) of U
---

<h2>Pointer Conversions</h2>

	Casting pointers to non-pointers and vice versa is allowed in D,
	however, do not do this for any pointers that point to data
	allocated by the garbage collector.

<h2>Implicit Conversions</h2>

	Implicit conversions are used to automatically convert
	types as required.
	<p>

	A typedef or enum can be implicitly converted to its base
	type, but going the other way requires an explicit
	conversion. For example:

-------------------
typedef int myint;
int i;
myint m;
i = m;			// OK
m = i;			// error
m = cast(myint)i;	// OK
-------------------

<h3>Integer Promotions</h3>

	Integer Promotions are conversions of the following types:
	<p>

	$(TABLE1
	<tr>
	<th> from
	<th> to
	<tr>
	<td> bool
	<td> int
	<tr>
	<td> byte
	<td> int
	<tr>
	<td> ubyte
	<td> int
	<tr>
	<td> short
	<td> int
	<tr>
	<td> ushort
	<td> int
	<tr>
	<td> char
	<td> int
	<tr>
	<td> wchar
	<td> int
	<tr>
	<td> dchar
	<td> uint
	)
	<p>

	If a typedef or enum has as a base type one of the types
	in the left column, it is converted to the type in the right
	column.

<h3>Usual Arithmetic Conversions</h3>

	The usual arithmetic conversions convert operands of binary
	operators to a common type. The operands must already be
	of arithmetic types.
	The following rules are applied
	in order, looking at the base type:

    $(OL
	<li> If either operand is real, the other operand is
	converted to real.

	<li> Else if either operand is double, the other operand is
	converted to double.

	<li> Else if either operand is float, the other operand is
	converted to float.

	<li> Else the integer promotions are done on each operand,
	followed by:

	$(OL 
	    <li> If both are the same type, no more conversions are done.

	    <li> If both are signed or both are unsigned, the
	    smaller type is converted to the larger.

	    <li> If the signed type is larger than the unsigned
	    type, the unsigned type is converted to the signed type.

	    <li> The signed type is converted to the unsigned type.
	)
    )

	$(P If one or both of the operand types is a typedef or enum after
	undergoing the above conversions, the result type is:)

	$(OL
	$(LI If the operands are the same type, the result will be the
	that type.)
	$(LI If one operand is a typedef or enum and the other is the base type
	of that typedef or enum, the result is the base type.)
	$(LI If the two operands are different typedefs or enums but of the same
	base type, then the result is that base type.)
	)

	$(P Integer values cannot be implicitly converted to another
	type that cannot represent the integer bit pattern after integral
	promotion. For example:)

---
ubyte  u1 = cast(byte)-1;   // error, -1 cannot be represented in a ubyte
ushort u2 = cast(short)-1;  // error, -1 cannot be represented in a ushort
uint   u3 = cast(int)-1;    // ok, -1 can be represented in a uint
ulong  u4 = cast(ulong)-1;  // ok, -1 can be represented in a ulong
---

	Floating point types cannot be implicitly converted to
	integral types.
	<p>

	Complex floating point types cannot be implicitly converted
	to non-complex floating point types.
	<p>

	Imaginary floating point types cannot be implicitly converted to
	float, double, or real types. Float, double, or real types
	cannot be implicitly converted to imaginary floating
	point types.

<h2>bool</h2>

	The bool type is a 1 byte size type that can only hold the
	value $(D_KEYWORD true) or $(D_KEYWORD false).
	The only operators that can accept operands of type bool are:
	&amp; | ^ &amp;= |= ^= ! &amp;&amp; || ?:.
	A bool value can be implicitly converted to any integral type,
	with $(D_KEYWORD false) becoming 0 and $(D_KEYWORD true) becoming 1.
	The numeric literals 0 and 1 can be implicitly
	converted to the bool values $(D_KEYWORD false) and $(D_KEYWORD true),
	respectively.
	Casting an expression to bool means testing for 0 or !=0 for
	arithmetic types, and $(D_KEYWORD null) or !=$(D_KEYWORD null)
	for pointers or references.


<h2><a name="delegates">Delegates</a></h2>

	There are no pointers-to-members in D, but a more useful
	concept called $(I delegates) are supported.
	Delegates are an aggregate of two pieces of data: an
	object reference and a function pointer. The object reference
	forms the $(I this) pointer when the function is called.
	<p>

	Delegates are declared similarly to function pointers,
	except that the keyword $(B delegate) takes the place
	of (*), and the identifier occurs afterwards:

-------------------
int function(int) fp;	// fp is pointer to a function
int delegate(int) dg;	// dg is a delegate to a function
-------------------

	The C style syntax for declaring pointers to functions is
	also supported:

-------------------
int (*fp)(int);		// fp is pointer to a function
-------------------

	A delegate is initialized analogously to function pointers:

-------------------
int func(int);
fp = &func;		// fp points to func

class OB
{   int member(int);
}
OB o;
dg = &o.member;		// dg is a delegate to object $(I o) and
			// member function $(I member)
-------------------

	Delegates cannot be initialized with static member functions
	or non-member functions.
	<p>

	Delegates are called analogously to function pointers:

-------------------
fp(3);		// call func(3)
dg(3);		// call o.member(3)
-------------------

)

Macros:
	TITLE=D Programming Language - Types
	WIKI=Type

